@@ -1,18 +1,21 @@
 package Lemonldap::NG::Common::Conf::File;
 
 use strict;
-use Lemonldap::NG::Common::Conf::Constants;
+use Lemonldap::NG::Common::Conf::Constants;    #inherits
+use Lemonldap::NG::Common::Conf::Serializer;
 
-our $VERSION = 0.22;
+our $VERSION = '0.99';
 
 sub prereq {
     my $self = shift;
     unless ( $self->{dirName} ) {
-        $Lemonldap::NG::Common::Conf::msg = '"dirName" is required in "File" configuration type !';
+        $Lemonldap::NG::Common::Conf::msg =
+          '"dirName" is required in "File" configuration type !';
         return 0;
     }
     unless ( -d $self->{dirName} ) {
-        $Lemonldap::NG::Common::Conf::msg = "Directory \"$self->{dirName}\" does not exist !";
+        $Lemonldap::NG::Common::Conf::msg =
+          "Directory \"$self->{dirName}\" does not exist !";
         return 0;
     }
     1;
@@ -35,12 +38,13 @@ sub lastCfg {
 
 sub lock {
     my $self = shift;
-    if( $self->isLocked ) {
+    if ( $self->isLocked ) {
         sleep 2;
-        return 0 if( $self->isLocked );
+        return 0 if ( $self->isLocked );
     }
-    unless( open F, ">".$self->{dirName} . "/lmConf.lock" ) {
-        $Lemonldap::NG::Common::Conf::msg = "Unable to lock (".$self->{dirName}."/lmConf.lock)\n";
+    unless ( open F, ">" . $self->{dirName} . "/lmConf.lock" ) {
+        $Lemonldap::NG::Common::Conf::msg =
+          "Unable to lock (" . $self->{dirName} . "/lmConf.lock)\n";
         return 0;
     }
     print F $$;
@@ -60,18 +64,21 @@ sub unlock {
 
 sub store {
     my ( $self, $fields ) = @_;
+    $fields = $self->serialize($fields);
     my $mask = umask;
-    umask ( oct ( '0027' ) );
-    unless( open FILE, '>' . $self->{dirName} . "/lmConf-" . $fields->{cfgNum} ) {
+    umask( oct('0027') );
+    unless ( open FILE,
+        '>' . $self->{dirName} . "/lmConf-" . $fields->{cfgNum} )
+    {
         $Lemonldap::NG::Common::Conf::msg = "Open file failed: $!";
         $self->unlock;
         return UNKNOWN_ERROR;
     }
-    while ( my ( $k, $v ) = each(%$fields) ) {
-        print FILE "$k\n\t$v\n\n";
+    foreach my $k ( sort keys %$fields ) {
+        print FILE "$k\n\t$fields->{$k}\n\n";
     }
     close FILE;
-    umask( $mask );
+    umask($mask);
     $self->unlock;
     return $fields->{cfgNum};
 }
@@ -93,12 +100,13 @@ sub load {
         }
     }
     close FILE;
-    return $f;
+    return $self->unserialize($f);
 }
 
 sub delete {
     my ( $self, $cfgNum ) = @_;
-    unlink ( $self->{dirName} . "/lmConf-$cfgNum" );
+    unlink( $self->{dirName} . "/lmConf-$cfgNum" );
 }
 
+1;
 __END__
