@@ -9,9 +9,10 @@ package Lemonldap::NG::Common::Crypto;
 use strict;
 use Crypt::Rijndael;
 use MIME::Base64;
+use bytes;
 use base qw(Crypt::Rijndael);
 
-our $VERSION = '0.2';
+our $VERSION = '0.99';
 
 our $msg;
 
@@ -34,8 +35,11 @@ sub encrypt {
     my $tmp;
     eval {
         $tmp = encode_base64(
-            $self->SUPER::encrypt( $str . "\0" x ( 16 - length($str) % 16 ) ),
-            '' );
+            $self->SUPER::encrypt(
+                $str . "\0" x ( 16 - bytes::length($str) % 16 )
+            ),
+            ''
+        );
     };
     if ($@) {
         $msg = "Crypt::Rijndael error : $@";
@@ -66,7 +70,7 @@ sub decrypt {
         $msg = '';
 
         # Obscure Perl re bug...
-        $tmp .="\0";
+        $tmp .= "\0";
         $tmp =~ s/\0*$//;
         return $tmp;
     }
